<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 12/09/2003 *
 * ===================                          Last change: 02/12/2006 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : what-mediadata.php                               *
 * -------------------------------------------------------------------- *
 * Short description : Media data                                       *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Mediendaten                                      *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2013 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	exit();
} // END - if

// Add description as navigation point
addYouAreHereLink('guest', __FILE__);

// Both ext-mediadata and ext-user must be there
if ((!isExtensionActive('mediadata')) && (!isAdmin())) {
	displayMessage('{%pipe,generateExtensionInactiveNotInstalledMessage=mediadata%}');
	return;
} elseif ((!isExtensionActive('user')) && (!isAdmin())) {
	displayMessage('{%pipe,generateExtensionInactiveNotInstalledMessage=user%}');
	return;
}

// Get total confirmed user ids...
$content['user_confirmed']   = getTotalConfirmedUser();
$content['user_unconfirmed'] = getTotalUnconfirmedUser();
$content['user_locked']      = getTotalLockedUser();
$content['user_count']       = ($content['user_confirmed'] + $content['user_unconfirmed'] + $content['user_locked']);

// Start of this exchange
$content['mt_start'] = generateDateTime(getMtStart(), '3');

// Project timestamp when number of members are reached
$PROJECTED = '0';
if ($content['user_count'] > 0) {
	// @TODO Find a better formular than this one
	$PROJECTED = round((time() - getMtStart()) / $content['user_count'] * getMtStage() + getMtStart());
} // END - if

// Generate timestamp
$TEST = makeTime(0, 0, 0, $PROJECTED);

if ($TEST > time()) {
	$content['projected'] = '{%message,USER_MT_PROJECTED=' . generateDateTime($PROJECTED, '3') . '%}';
} else {
	$content['projected'] = getMaskedMessage('USER_MT_PROJECTED', '{--USER_PROJECTION_UNKNOWN--}');
}

// User who can receive mails
$content['user_max_mails'] = countSumTotalData('CONFIRMED', 'user_data', 'max_mails', 'status', TRUE, runFilterChain('user_exclusion_sql', " AND `max_mails` > 0"));

// Users who can receive mails today
$content['user_max_rec'] = countSumTotalData('CONFIRMED', 'user_data', 'receive_mails', 'status', TRUE, runFilterChain('user_exclusion_sql', " AND `receive_mails` > 0"));

// Max mails per day
$content['max'] = countSumTotalData('CONFIRMED', 'user_data', 'max_mails', 'status', FALSE, runFilterChain('user_exclusion_sql', " AND `max_mails` > 0"));

// Max mails for this day
$content['rec'] = countSumTotalData('CONFIRMED', 'user_data', 'receive_mails', 'status', FALSE, runFilterChain('user_exclusion_sql', " AND `receive_mails` > 0"));

// Initial lots of variables
$bmails = '0';
$sent   = '0';
$max    = '0';
$rec    = '0';
$clicks = '0';

// Mail orders (only current)
$nmails = getMediadataEntry('normal_orders');
if (empty($nmails)) $nmails = '0';

// Mails sent so far
$clicks = getMediadataEntry('normal_clicks');
$sent   = getMediadataEntry('normal_send');
if (empty($sent))   $sent   = '0';
if (empty($clicks)) $clicks = '0';

// Bonus mails
if (isExtensionActive('bonus')) {
	$bmails = getMediadataEntry('bonus_orders');
	if (empty($bmails)) $bmails = '0';

	// Count bonus mails clicks / sent mails
	$dmy  = getMediadataEntry('bonus_clicks');
	$dmy2 = getMediadataEntry('bonus_send');
	if (empty($dmy))  $dmy  = '0';
	if (empty($dmy2)) $dmy2 = '0';
	$sent += $dmy2; $clicks += $dmy;
} // END - if

$content['user_stats'] = ($nmails + $bmails);
$content['sent']       = $sent;

// All clicks
$content['user_links'] = abs($sent - $clicks);

// Click rate
$content['click_rate'] = calculatePercentageRate($clicks, $sent);

// Load jackpot
$content['jackpot'] = '0';
if (isExtensionActive('jackpot')) $content['jackpot'] = getJackpotPoints();
if (empty($content['jackpot'])) $content['jackpot'] = '0';

// Total referral link clicks, total logins
$result = sqlQuery('SELECT
	SUM(`d`.`ref_clicks`) AS `total_ref_clicks`,
	SUM(`d`.`total_logins`) AS `total_logins`
FROM
	`{?_MYSQL_PREFIX?}_user_data` AS `d`
' . runFilterChain('user_exclusion_sql', ''), __FILE__, __LINE__);
list($ref, $logins) = sqlFetchRow($result);
sqlFreeResult($result);

// Get total points
$points = getMediadataEntry('total_points');

if (empty($points)) $points = '0';
if (empty($ref))    $ref    = '0';
if (empty($logins)) $logins = '0';

$content['total_points']    = $points;
$content['total_refclicks'] = $ref;
$content['total_logins']    = $logins;

// Referral banner
$total = countSumTotalData('Y', 'refbanner', 'id', 'visible', TRUE);

// Total views and clicks
$result = sqlQuery("SELECT SUM(`counter`) AS `views`, SUM(`clicks`) AS `clicks` FROM `{?_MYSQL_PREFIX?}_refbanner` WHERE `visible`='Y'", __FILE__, __LINE__);
list($views, $clicks) = sqlFetchRow($result);
sqlFreeResult($result);

if (empty($total))  $total  = '0';
if (empty($views))  $views  = '0';
if (empty($clicks)) $clicks = '0';

$content['ref_total']  = $total;
$content['ref_views']  = $views;
$content['ref_clicks'] = $clicks;

//
// Extra mediadata depending on installed extensions
//

// Initialize Variables
$OUT_EXTRA   = '';
$OUT_POINTS  = '';
$OUT_USER    = '';
$OUT_SPECIAL = ''; // We don't need row count here

// @TODO Rewrite all these if-blocks to filters
if (isExtensionActive('beg')) {
	// Clicks on beg links
	$data = array(
		'lang'  => '{--MEDIA_BEG_CLICKS--}',
		'value' => countSumTotalData('0', 'user_data', 'beg_clicks', 'userid', FALSE, ' WHERE `beg_clicks` > 0')
	);
	$OUT_EXTRA .= loadTemplate('mediadata_extra_row', TRUE, $data);
} // END - if

if (isExtensionActive('doubler')) {
	// Add header
	$OUT_SPECIAL .= loadTemplate('mediadata_extra_hrow', TRUE, '{--MEDIA_DOUBLER_HEADER--}');

	// Add first line (count)
	$data = array(
		'lang'  => '{--MEDIA_DOUBLER_COUNT--}',
		'value' => countSumTotalData('Y','doubler','id','completed', TRUE, ' AND `points` > 0'),
	);

	// Add points
	$OUT_SPECIAL .= loadTemplate('mediadata_extra_row2', TRUE, $data);
	$data = array(
		'lang'  => '{--MEDIA_DOUBLER_CPOINTS--}',
		'value' => countSumTotalData('Y','doubler','id','completed', FALSE, ' AND `points` > 0')
	);
	$OUT_SPECIAL .= loadTemplate('mediadata_extra_row', TRUE, $data);
	$data = array(
		'lang'  => '{--MEDIA_DOUBLER_WAITING--}',
		'value' => countSumTotalData('N','doubler','id','completed', TRUE, ' AND `points` > 0')
	);
	$OUT_SPECIAL .= loadTemplate('mediadata_extra_row', TRUE, $data);
	$data = array(
		'lang'  => '{--MEDIA_DOUBLER_WPOINTS--}',
		'value' => countSumTotalData('N','doubler','id','completed', FALSE, ' AND `points` > 0')
	);
	$OUT_SPECIAL .= loadTemplate('mediadata_extra_row', TRUE, $data);
} // END - if

if (isExtensionActive('holiday')) {
	// Total holiday requests
	$result = sqlQuery("SELECT
	COUNT(`h`.`userid`) AS `count_holidays`
FROM
	`{?_MYSQL_PREFIX?}_user_holidays` AS `h`
LEFT JOIN
	`{?_MYSQL_PREFIX?}_user_data` AS `d`
ON
	`h`.`userid`=`d`.`userid`
WHERE
	`d`.`holiday_active`='Y'
" . runFilterChain('user_exclusion_sql', ' '),
	__FILE__, __LINE__);
	list($holiday) = sqlFetchRow($result);
	sqlFreeResult($result);
	if (empty($holiday)) $holiday = '0';
	$data = array(
		'lang'  => '{--MEDIA_HOLIDAY_COUNT--}',
		'value' => $holiday
	);
	$OUT_USER .= loadTemplate('mediadata_extra_row', TRUE, $data);
} // END - if

if (isExtensionActive('transfer')) {
	// Statistics for points transfers
	$result = sqlQuery("SELECT
	COUNT(`t`.`id`) AS `cnt`,
	SUM(`t`.`points`) AS `points`
FROM
	`{?_MYSQL_PREFIX?}_user_transfers_in` AS `t`
LEFT JOIN
	`{?_MYSQL_PREFIX?}_user_data` AS `d`
ON
	`t`.`userid`=`d`.`userid`
" . runFilterChain('user_exclusion_sql', ''),
		__FILE__, __LINE__);
	list($count_in, $points_in) = sqlFetchRow($result);
	sqlFreeResult($result);
	if (empty($count_in))  $count_in  = '0';
	if (empty($points_in)) $points_in = '0';

	$result = sqlQuery("SELECT
	COUNT(`t`.`id`) AS `cnt`, SUM(`t`.`points`) AS `points`
FROM
	`{?_MYSQL_PREFIX?}_user_transfers_out` AS `t`
LEFT JOIN
	`{?_MYSQL_PREFIX?}_user_data` AS `d`
ON
	`t`.`userid`=`d`.`userid`
" . runFilterChain('user_exclusion_sql', ''),
	__FILE__, __LINE__);
	list($count_out, $points_out) = sqlFetchRow($result);
	sqlFreeResult($result);
	if (empty($count_out))  $count_out  = '0';
	if (empty($points_out)) $points_out = '0';

	// Add header
	$OUT_SPECIAL .= loadTemplate('mediadata_extra_hrow', TRUE, '{--MEDIA_TRANSFER_HEADER--}');
	$data = array(
		'lang'  => '{--MEDIA_TRANSFER_IN_COUNT--}',
		'value' => $count_in,
	);
	$OUT_SPECIAL .= loadTemplate('mediadata_extra_row2', TRUE, $data);
	$data = array(
		'lang'  => '{--MEDIA_TRANSFER_IN_POINTS--}',
		'value' => $points_in
	);
	$OUT_SPECIAL .= loadTemplate('mediadata_extra_row', TRUE, $data);
	$data = array(
		'lang'  => '{--MEDIA_TRANSFER_OUT_COUNT--}',
		'value' => $count_out
	);
	$OUT_SPECIAL .= loadTemplate('mediadata_extra_row', TRUE, $data);
	$data = array(
		'lang'  => '{--MEDIA_TRANSFER_OUT_POINTS--}',
		'value' => $points_out
	);
	$OUT_SPECIAL .= loadTemplate('mediadata_extra_row', TRUE, $data);
} // END - if

$content['out_extra']   = $OUT_EXTRA;
$content['out_points']  = $OUT_POINTS;
$content['out_user']    = $OUT_USER;
$content['out_special'] = $OUT_SPECIAL;

// Load template
loadTemplate('mediadata', FALSE, $content);

// [EOF]
?>
