<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 11/16/2003 *
 * ===================                          Last change: 12/13/2004 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : mysql-connect.php                                *
 * -------------------------------------------------------------------- *
 * Short description : Connects to your database                        *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Verbindet zu Ihrer Datenbank                     *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2013 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Init array
$__functions = array();

// Init include file array as it follows same naming scheme
foreach ( array(
		'stats',
		'xml',
		'callback',
		'referral',
		'email',
		'request',
		'session',
		'code',
		'pool',
		'language',
		'sql',
		'expression',
		'filter',
		'extensions') as $lib) {

	// Add it
	array_push($__functions, $lib . '-functions');
} // END - foreach

// Load more function libraries or includes
foreach (array_merge($__functions, array('filters', 'mysql-manager', 'handler')) as $lib) {
	// Load special functions
	loadIncludeOnce('inc/' . $lib . '.php');
} // END - foreach

// Remove array
unset($__functions);

// Set error handler
set_error_handler('__errorHandler');

// Disable block-mode by default
enableBlockMode(FALSE);

// Init error handler
initErrorHandler();

// Init request
initRequest();

// Init userid
initMemberId();

// Set important header_sent
if (!isset($GLOBALS['__header_sent'])) {
	$GLOBALS['__header_sent'] = '0';
} // END - if

// Init fatal messages
initFatalMessages();

// Enable HTML templates by default
enableTemplateHtml();

// Are we in installation phase?
if ((!isInstaller()) && (isInstalled())) {
	// Load configuration file(s) here
	loadIncludeOnce('inc/load_config.php');

	// Load database layer here
	loadIncludeOnce('inc/db/lib.php');

	// Init message system
	initMessages();

	// CSS array
	initExtensionCssFiles();

	// Initialize SQL link
	initSqlLink();
} else {
	// Default output is 'direct' for HTML output
	setConfigEntry('OUTPUT_MODE', 'direct');

	// This hack prevents a backtrace in CSS output
	if (isCssOutputMode()) {
		// Problem with config so set output mode
		setConfigEntry('OUTPUT_MODE', 'render');
	} // END - if

	// CFG: DATABASE-TYPE
	setConfigEntry('_DB_TYPE', 'mysql');

	// Set link as down
	unsetSqlLinkUp(__FILE__, __LINE__);

	// Load database layer here
	loadIncludeOnce('inc/db/lib.php');

	// Init message system
	initMessages();

	// Init session
	initSession();

	// Include more
	foreach (array('databases', 'install-functions', 'load_config', 'load_cache') as $inc) {
		// Load include file
		loadIncludeOnce('inc/' . $inc . '.php');
	} // END - foreach

	// Init installer
	initInstaller();

	// Check whether we are in installation routine
	if ((!isInstalling()) && (!isCssOutputMode()) && (!isRawOutputMode())) {
		// Redirect to the URL
		redirectToUrl('install.php');
	} // END - if
}

// Handle fatal errors
runFilterChain('handle_fatal_errors');

// [EOF]
?>
